package model

import (
	"fmt"
	"kpt-pasture/store/kptstore"

	pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"
)

type EventEstrus struct {
	Id               int64                           `json:"id"`
	PastureId        int64                           `json:"pastureId"`
	NeckRingNumber   string                          `json:"neckRingNumber"`
	CowId            int64                           `json:"cowId"`
	Lact             int32                           `json:"lact"`
	ExposeEstrusType pasturePb.ExposeEstrusType_Kind `json:"exposeEstrusType"`
	EstrusStartDate  string                          `json:"estrusStartDate"`
	ActiveDate       string                          `json:"activeDate"`
	LastEstrusDate   string                          `json:"lastEstrusDate"`
	Level            pasturePb.EstrusLevel_Kind      `json:"level"`
	IsPeak           pasturePb.IsShow_Kind           `json:"isPeak"`
	DayHigh          int32                           `json:"dayHigh"`
	MaxHigh          int32                           `json:"maxHigh"`
	Result           pasturePb.EstrusResult_Kind     `json:"result"`
	Remarks          string                          `json:"remarks"`
	IsShow           pasturePb.IsShow_Kind           `json:"isShow"`
	OperationId      int64                           `json:"operationId"`
	OperationName    string                          `json:"operationName"`
	MessageId        int64                           `json:"messageId"`
	MessageName      string                          `json:"messageName"`
	CreatedAt        int64                           `json:"createdAt"`
	UpdatedAt        int64                           `json:"updatedAt"`
}

func (e *EventEstrus) TableName() string {
	return "event_estrus"
}

func NewEventEstrus(
	pastureId int64,
	exposeEstrusType pasturePb.ExposeEstrusType_Kind,
	level pasturePb.EstrusLevel_Kind,
	result pasturePb.EstrusResult_Kind,
	isShow, isPerk pasturePb.IsShow_Kind,
	lastEstrusDate, activeDate string,
	dayHigh, maxHigh int32,
	cow *Cow,
) *EventEstrus {
	return &EventEstrus{
		PastureId:        pastureId,
		NeckRingNumber:   cow.NeckRingNumber,
		CowId:            cow.Id,
		Lact:             cow.Lact,
		ExposeEstrusType: exposeEstrusType,
		LastEstrusDate:   lastEstrusDate,
		ActiveDate:       activeDate,
		Level:            level,
		IsShow:           isShow,
		Result:           result,
		DayHigh:          dayHigh,
		MaxHigh:          maxHigh,
		IsPeak:           isPerk,
	}
}

type EstrusSlice []*EventEstrus

func (e EstrusSlice) ToPB(
	dB *kptstore.DB,
	getCowInfo func(dB *kptstore.DB, cowId int64) *Cow,
	getCowLastEvent func(DB *kptstore.DB, cowId int64, eventCategoryId pasturePb.EventCategory_Kind) *EventCowLog,
) []*pasturePb.EstrusItems {
	res := make([]*pasturePb.EstrusItems, len(e))
	for i, v := range e {
		cowInfo := getCowInfo(dB, v.CowId)
		lastEventLog := getCowLastEvent(dB, v.CowId, pasturePb.EventCategory_Breed)
		planDay, optimumMatingTime := "", ""
		lastBreedEvnetDetails := ""
		if lastEventLog != nil {
			lastBreedEvnetDetails = fmt.Sprintf("%s %s", lastEventLog.EventTypeName, lastEventLog.EventDescription)
		}
		res[i] = &pasturePb.EstrusItems{
			Id:                     int32(v.Id),
			CowId:                  int32(v.CowId),
			DayAge:                 cowInfo.DayAge,
			Lact:                   cowInfo.Lact,
			Level:                  v.Level,
			PenName:                cowInfo.PenName,
			Status:                 v.IsShow,
			CalvingAge:             int32(cowInfo.CalvingAge),
			PlanDay:                planDay,
			MatingTimes:            cowInfo.MatingTimes,
			OptimumMatingStartTime: optimumMatingTime,
			OptimumMatingEndTime:   optimumMatingTime,
			LastBreedEvnetDetails:  lastBreedEvnetDetails,
		}
	}
	return res
}
